import sys
import logging
import argparse
from typing import List

from hooks.rule import ValidationResult, Rule
from hooks.kubernetes_secret import KubernetesSecretRule

logging.basicConfig(
    format="%(asctime)s - %(levelname)s - %(message)s", level=logging.INFO
)


def main():
    args = parse_arguments()

    rules: List[Rule] = [KubernetesSecretRule()]
    only_allowed_rules = [r for r in rules if r.name() not in args.exclude]

    if len(only_allowed_rules):
        logging.info("Validating the following rules:")
        for rule in only_allowed_rules:
            logging.info(f"\u2713 {rule.name()}")
    else:
        logging.warning("No rules activated to validate files")

    try:
        apply_rules_to_files(args.files, only_allowed_rules)
        logging.info("No validation errors found")
    except Exception as e:
        logging.error(e)
        sys.exit(1)


def parse_arguments():
    parser = argparse.ArgumentParser()
    parser.add_argument("--exclude", action="append", default=[])
    parser.add_argument("files", nargs="+")
    return parser.parse_args()


def apply_rules_to_files(files: List[str], rules: List[Rule]):
    for rule in rules:
        result: ValidationResult = rule.validate(files)
        if not result.valid:
            raise Exception(result.error)



from typing import List, Optional
from abc import ABCMeta, abstractmethod
from dataclasses import dataclass


@dataclass
class ValidationResult:
    """Result of a rule that has been validated."""

    valid: bool
    error: Optional[str]


class Rule:
    """Custom rule that will validate if files are okay to commit."""

    __metaclass__ = ABCMeta

    @staticmethod
    def name(self) -> str:
        return "default-rule"

    @abstractmethod
    def validate(self, files: List[str]) -> ValidationResult:
        raise NotImplementedError



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pre-commit-k8s",
    version="0.0.2",
    author="Martijn Zwennes",
    author_email="martijnzwennes@pm.me",
    description="pre-commit rules for Kubernetes related files",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zwennesm/pre-commit-k8s",
    packages=setuptools.find_packages(include=["hooks", "hooks.*"]),
    entry_points={"console_scripts": ["pre-commit-k8s=hooks.validate:main"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)



from typing import List
from hooks.rule import Rule, ValidationResult


class KubernetesSecretRule(Rule):
    """Check if commit contains Kubernetes Secret object."""

    @staticmethod
    def name() -> str:
        return "kubernetes-secrets"

    def validate(self, files: List[str]) -> ValidationResult:

        for file_name in files:
            with open(file_name, "r") as current:
                error_msg = f"File contains a Kubernetes Secret: {file_name}"
                if "kind: Secret" in current.read():
                    return ValidationResult(False, error_msg)

                return ValidationResult(True, None)
